package de.tu_darmstadt.tk.SmartHomeNetworkSim.core;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Manages the Simulation by calling the relevant methods.
 *
 * @author Andreas T. Meyer-Berg
 */
public class SimulationManager {
	/**
	 * Model which should be simulated
	 */
	Model model;
	
	/**
	 * True if packets should be printed
	 */
	boolean printPackets = false;
	
	/**
	 * Whether the model status changed during the last simulation and the panel should be repainted
	 */
	boolean statusChanged = false;
	
	/**
	 * Writer to write the packets to a file
	 */
	BufferedWriter writer;
	
	/**
	 * Creates a new Simulationmanager
	 * 
	 * @param model Model that should be simulated
	 */
	public SimulationManager(Model model) {
		this.model = model;
	}

	/**
	 * Simulates one time step at a given time fur a given duration
	 * 
	 * @param startTime
	 *            Time the simulation interval starts in
	 *            System.currentTimeMillis() time
	 * @param duration
	 *            Duration of the simulation interval in milliseconds
	 */
	public void simulateTimeIntervall(long startTime, long duration) {
		//Nothing changed so far
		statusChanged = false;
		//Simulate all Links, and their connections
		model.getConnectionNetworks().forEach(d -> {
			d.simulateTimeInterval(startTime, duration);
			if(d.getStatusChanged()){
				for(Connection c:d.getConnections()){
					if(c.getStatus()==Connection.DONE){
						model.getConnections().remove(c);
					}
				}
				statusChanged = true;
			}
		});
		//Simulate SmartDevices - if they need some logic
		model.getDevices().forEach(d -> d.simulateTimeStep(startTime, duration));
		//Store Packages/Export Packages etc. (for debug purposes)
		if(statusChanged){
			model.setChanged();
			model.notifyObservers();
		}
		if(printPackets){
			try {
				File f = new File("testPackets.log");
				f.createNewFile();
				writer = new BufferedWriter(new FileWriter(f.getAbsolutePath(),true));
				model.getConnectionNetworks().forEach(d->d.getPackets()
						.forEach(p-> 
						{
							try {
								if(p == null)
									writer.append("Packet: Null\n");
								else 
									writer.append(p.getTextualRepresentation()+"\n");								
							} catch (Exception e) {
								// TODO: handle exception
							}
						}));				
			} catch (Exception e) {
				// TODO: handle exception
			} finally {
				if(writer != null)
					try {
						writer.close();
					} catch (IOException e) {}				
			}
		}
		
		
		
		
	}

	/**
	 * Sets Print Packets, if true, the simulation will print the packets
	 * @param printPackets true if simulation should print the packets
	 */
	public void setPrintPackets(boolean printPackets) {
		this.printPackets = printPackets;
	}

	/**
	 * Returns true if the simulations will print the packets
	 * 
	 * @return if the packets are printed
	 */
	public boolean getPrintPackets() {
		return printPackets;
	}
}
