package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.LinkedList;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.event.MouseInputListener;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.ConnectionImplementation;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Packet;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleLink;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleProtocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.util.Pair;

/**
 * Listener which detects User Interaction with the {@link VisualisationPanel},
 * stores interaction information, triggers the controller and helps
 * visualization of the interaction
 * 
 * @author Andreas T. Meyer-Berg
 */
public class VisualisationInteractor implements MouseInputListener,
		MouseMotionListener, ActionListener, KeyListener {

	/**
	 * Smart Home model which should be interacted with
	 */
	private Model model;

	/**
	 * Controller to notify when the model changes
	 */
	private Controller controller;

	/**
	 * Panel which is observed
	 */
	private VisualisationPanel panel;

	/**
	 * Menu which is shown on right clicks
	 */
	private JPopupMenu rightClickMenu;

	/**
	 * RightClick MenuItem for SmartDevice creation
	 */
	private JMenuItem itemCreate;

	/**
	 * RightClick MenuItem for Link creation
	 */
	private JMenuItem itemCreateLink;
	
	/**
	 * RightClick MenuItem for Connection creation
	 */
	private JMenuItem itemCreateConnection;
	/**
	 * RightClickMenu for editing connections
	 */
	private JMenu itemEditConnection;
	/**
	 * Position of editCreate connection in the Rightclick menu 
	 */
	private int editCreateConnectionIndex = 2;

	/**
	 * RightClick MenuItem for SmartDevice deletion
	 */
	private JMenuItem itemDelete;
	
	/**
	 * RightClick MenuItem for deletion of multiple selected SmartDevices
	 */
	private JMenuItem itemDeleteSelected;

	/**
	 * RightClick MenuItem for debug purposes
	 */
	private JMenuItem itemDebug;

	/**
	 * SmartDevice that is dragged on Screen
	 */
	public SmartDevice dragged = null;

	/**
	 * Current x Position of the dragged device
	 */
	public int dragged_x;

	/**
	 * Current y Position of the dragged device
	 */
	public int dragged_y;
	
	/**
	 * x Position where the drage begun
	 */
	public int dragged_x_start;
	
	/**
	 *  y Position where the drag begun
	 */
	public int dragged_y_start;
	
	/**
	 * Devices which are currently selected
	 */
	public LinkedList<SmartDevice> selectedDevices = new LinkedList<SmartDevice>();
	public LinkedList<SmartDevice> selectedDevicesDrag = new LinkedList<SmartDevice>();
	private boolean draggedDeviceWasSelected = false;
	
	public static final byte NOTHING = 0; 
	public static final byte RIGHTCLICK_MENU = 1; 
	public static final byte SELECTED_DRAG = 2;
	public static final byte DRAG_CONNECTION = 3;
	public static final byte DRAG_SELECT = 4; 
	public static final byte SELECTED = 5;
	/**
	 * Mode, which action is currently executed
	 */
	public byte mode = NOTHING;
	
	/**
	 * SmartDevice that was clicked
	 */
	private SmartDevice clicked = null;
	
	/**
	 * Clicked Connections
	 */
	public LinkedList<Pair<Connection,Pair<Port,Port>>> clickedConnection = new LinkedList<Pair<Connection,Pair<Port,Port>>>();

	public SmartDevice connectionFrom = null;

	/**
	 * True if the ControlKey is currently pressed
	 */
	public boolean controlDown = false;

	/**
	 * Creates a new VisualisationInteractor
	 *
	 * @param model
	 *            Model which is visualized
	 * @param controller
	 *            controller that is accessed
	 * @param panel
	 *            which should visualize the interactions
	 */
	public VisualisationInteractor(Model model, Controller controller,
			VisualisationPanel panel) {
		// Initialize the values
		this.model = model;
		this.controller = controller;
		this.panel = panel;

		initializeRightClickMenu();
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		/*
		 * FindSmartDevice that was clicked
		 */
		clicked = getSmartDeviceAtPosition(e.getX(), e.getY());
		clickedConnection.clear();
		if(clicked == null)
			clickedConnection = getConnectionsAtPosition(e.getX(), e.getY());
		
		
		switch (mode) {
		case DRAG_CONNECTION:
			finishConnectionCreation();			
			break;
		case SELECTED_DRAG:
			// Finish Drag Device operations
			finishDrag();
		case SELECTED:			
		case NOTHING:
			// If Rightclick
			if (e.getButton() == MouseEvent.BUTTON3) {
				// Show the RightClickMenu
				showRightClickMenu(clicked);
				break;
			}
			if(!selectedDevices.isEmpty()&&!controlDown){
				selectedDevices.clear();
				if (clicked == null) {
					panel.repaint();					
				}
			}
			if(clicked!=null){
				if(!controlDown)
					selectedDevices.add(clicked);
				else if(selectedDevices.remove(clicked)&&!draggedDeviceWasSelected){
					selectedDevices.add(clicked);
				}
				mode = SELECTED;
				panel.repaint();
			}else{
				if(!controlDown)
					mode = NOTHING; 
				else if(!selectedDevices.isEmpty())
					mode = SELECTED;

				panel.repaint();	
			}
			break;
		default:
			break;
		}
		
		

	}

	@Override
	public void mousePressed(MouseEvent e) {
		// Save mouse position for right Click options
		dragged_x = e.getX();
		dragged_y = e.getY();
		// Check if SmartDevice was clicked
		SmartDevice pressedOn = getSmartDeviceAtPosition(dragged_x, dragged_y);
		switch (mode) {
		case SELECTED:
			if(!controlDown)
				mode = NOTHING;
		case RIGHTCLICK_MENU:
		case NOTHING:
			if (pressedOn == null){
				//Click drag - multi select
				dragged_x_start = e.getX();
				dragged_y_start = e.getY();
				mode = DRAG_SELECT;
				if(!controlDown)
					selectedDevices.clear();
				break;
			} else if (e.getButton() == MouseEvent.BUTTON1){// && connectionFrom == null) {
				// Recognize possible drag operation
				draggedDeviceWasSelected = selectedDevices.contains(pressedOn);
				if(!draggedDeviceWasSelected){
					if(!controlDown)
						selectedDevices.clear();
					selectedDevices.add(pressedOn);
				}
				dragged = pressedOn;
				dragged_x = pressedOn.getX();
				dragged_y = pressedOn.getY();
				mode = SELECTED_DRAG;
			} else if (e.getButton() == MouseEvent.BUTTON3){// && dragged == null) {
				connectionFrom = pressedOn;
				mode = DRAG_CONNECTION;
			}
			break;
		case DRAG_SELECT:
			finishDragSelect();
			break;
		default:
			mode = NOTHING;
			break;
		}
		panel.repaint();
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// Finish operation
		switch (mode) {
		case DRAG_SELECT:
			finishDragSelect();
			panel.repaint();
			break;
		case SELECTED_DRAG:
			finishDrag();
			
			break;
		case DRAG_CONNECTION:
			finishConnectionCreation();
			break;
		default:
			break;
		}
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		// move dragged object/object selection on screen
		switch (mode) {
		case DRAG_SELECT:
		case DRAG_CONNECTION:
			//Update position
			if (e.getX() < 0)
				dragged_x = 0;
			else if (e.getX() >= model.getWidth())
				dragged_x = model.getWidth()-1;
			else
				dragged_x = e.getX();
			if (e.getY() < 0)
				dragged_y = 0;
			else if (e.getY() >= model.getHeight())
				dragged_y = model.getHeight()-1;
			else
				dragged_y = e.getY();
			if(mode == DRAG_CONNECTION)
				break;
			//Update Selected objects, if DragSelect mode is active
			int min_x = Math.min(dragged_x, dragged_x_start);
			int min_y = Math.min(dragged_y, dragged_y_start);
			int max_x = Math.max(dragged_x, dragged_x_start);
			int max_y = Math.max(dragged_y, dragged_y_start);
			//Remove unselected
			selectedDevicesDrag.removeIf(s->(s.getX()<min_x||s.getX()>max_x||s.getY()<min_y||s.getY()>max_y));
			//Add selected devices
			for(SmartDevice sel:model.getDevices()){
				if(sel.getX()>=min_x&&sel.getX()<=max_x&&sel.getY()>=min_y&&sel.getY()<=max_y&&!selectedDevicesDrag.contains(sel)){
					selectedDevicesDrag.add(sel);
				}
			}
			break;
		case SELECTED_DRAG:
			//new Position of the dragged Device
			dragged_x = e.getX();
			dragged_y = e.getY();
			//offset of the new Position - old position
			int x_offset = dragged_x - dragged.getX();
			int y_offset = dragged_y - dragged.getY();
			//Validate for all moved devices, that they are within the model, if not change offset
			for(SmartDevice d:selectedDevices){
				if (d.getX() + x_offset <= controller.getDevice_visualization_radius())
					x_offset = controller.getDevice_visualization_radius()-d.getX();
				else if (d.getX() + x_offset >= model.getWidth() - controller.getDevice_visualization_radius())
					x_offset = model.getWidth() - controller.getDevice_visualization_radius()-d.getX();
				if (d.getY() + y_offset <= controller.getDevice_visualization_radius())
					y_offset = controller.getDevice_visualization_radius()-d.getY();
				else if (d.getY() + y_offset >= model.getHeight() - controller.getDevice_visualization_radius())
					y_offset = model.getHeight() - controller.getDevice_visualization_radius()-d.getY();
			}
			//update the dragged position, if the offset changed
			dragged_x =dragged.getX() + x_offset;
			dragged_y =dragged.getY() + y_offset;
			break;

		default:
			break;
		}
		
		// repaint the dragged smartDevice or new connection
		if (mode == SELECTED_DRAG || mode == DRAG_CONNECTION || mode == DRAG_SELECT)
			panel.repaint();
	}
	
	/**
	 * Finishes the drag operation, if a SmartDevice was dragged, repaint the
	 * panel and set dragged to null
	 */
	public void finishDrag() {
		if (mode == SELECTED_DRAG){
			if(dragged != null
				&& (dragged.getX() != dragged_x || dragged.getY() != dragged_y)) {
				int x_offset = dragged_x-dragged.getX();
				int y_offset = dragged_y-dragged.getY();
				for(SmartDevice d: selectedDevices)
					controller.moveSmartDevice(d, d.getX()+x_offset, d.getY()+y_offset, d.getZ());
			}
			dragged = null;
			if(selectedDevices.isEmpty())
				mode = NOTHING;
			else
				mode = SELECTED;
			panel.repaint();
			model.notifyObservers();
		}
	}

	private void finishDragSelect() {
		LinkedList<SmartDevice> merged = new LinkedList<SmartDevice>();
		//XOR of selectedDevices and selectedDevices Drag
		merged.addAll(selectedDevices);
		merged.removeAll(selectedDevicesDrag);
		selectedDevicesDrag.removeAll(selectedDevices);
		merged.addAll(selectedDevicesDrag);
		//clear sets
		selectedDevices.clear();
		selectedDevicesDrag.clear();
		selectedDevices = merged;
		if(selectedDevices.isEmpty())
			mode = NOTHING;
		else
			mode = SELECTED;
	}
	
	/**
	 * Finishes the create connection operation and open a PopUp for
	 * Link/Connection creation
	 */
	private void finishConnectionCreation() {
		/**
		 * SmartDevice the connection was
		 */
		SmartDevice connectionTo = getSmartDeviceAtPosition(dragged_x,
				dragged_y);
		if (mode == DRAG_CONNECTION && connectionFrom != null && connectionTo != null
				&& connectionFrom != connectionTo) {
			// Create new Connection
			Link l = new SimpleLink("Ethernet: " + connectionFrom.getName()
					+ " to " + connectionTo.getName());
			l.addDevice(connectionFrom);
			l.addDevice(connectionTo);
			Port p1 = new Port(connectionFrom, (short) 1);
			Port p2 = new Port(connectionTo, (short) 2);
			Connection c = new ConnectionImplementation(l, new SimpleProtocol(p1, p2));
			c.addSmartDevice(p1);
			c.addSmartDevice(p2);
			p1.setConnection(c);
			p2.setConnection(c);
			connectionTo.addLink(l);
			connectionTo.addPort(p2);
			connectionFrom.addLink(l);
			connectionFrom.addPort(p1);
			l.addConnection(c);
			model.addConnection(c);
			
		}
		connectionFrom = null;
		if(selectedDevices.isEmpty())
			mode = NOTHING;
		else
			mode = SELECTED;
		panel.repaint();
		controller.notifyObservers();
	}

	/**
	 * Returns SmartDevice which is at position (x,y) or within its
	 * visualization radius. Returns null, if no SmartDevice is within the
	 * range.
	 * 
	 * @param x
	 *            x-position which should be checked
	 * @param y
	 *            y-position which should be checked
	 * @return {@link SmartDevice} at position (x,y) or null if there is no
	 */
	private SmartDevice getSmartDeviceAtPosition(int x, int y) {
		// Check is device is inside visualization radius
		for (SmartDevice d : model.getDevices()) {
			if (Math.abs(d.getX() - x) < controller.getDevice_visualization_radius()
					&& Math.abs(d.getY() - y) < controller.getDevice_visualization_radius()) {
				return d;
			}
		}
		return null;
	}
	
	/**
	 * Returns all Connections, which cross point (x,y)
	 * @param x xPosition of the point (x,y)
	 * @param y yPosition of the point (x,y)
	 * @return List of all possible clicked Connections
	 */
	private LinkedList<Pair<Connection, Pair<Port, Port>>> getConnectionsAtPosition(int x, int y) {
		LinkedList<Pair<Connection,Pair<Port,Port>>> edges = new LinkedList<Pair<Connection,Pair<Port,Port>>>();
		// Check is device is inside visualization radius
		for(Connection c: model.getConnections()){
			if(c.getProtocol()==null)
				continue;
			for(Pair<Port, Port> p: c.getProtocol().getTopology()){
				if(pointOnConnection(x,y,p)){
					edges.add(new Pair<Connection, Pair<Port,Port>>(c, p));
					//break;//Connection was clicked - check for further connections
				}
			}
		}
		return edges;
	}

	/**
	 * Returns true if the point (x,y) is on the connection part p.
	 * 
	 * @param x xPosition of the Point
	 * @param y yPosition of the Point
	 * @param p connection part, which connects to Ports with a line
	 * @return true if point is on connection, else false
	 */
	private boolean pointOnConnection(int x, int y, Pair<Port, Port> p) {
		//No valid connection part
		if(p.getLeft()==null||p.getRight()==null||p.getLeft().getOwner()==null||p.getRight().getOwner()==null)
			return false;
		int lX = p.getLeft().getOwner().getX();
		int lY = p.getLeft().getOwner().getY();
		int rX = p.getRight().getOwner().getX();
		int rY = p.getRight().getOwner().getY();
		//Point out of bounds
		if(x < Math.min(lX, rX) || x > Math.max(lX, rX) || y < Math.min(lY, rY) || y > Math.max(lY, rY))
			return false;
		double dLP = distance(lX,lY,x,y);
		double dPR = distance(x,y,rX,rY);
		double dLR = distance(lX,lY,rX,rY);
		double diff = Math.abs(dLP+dPR-dLR);
		return diff < 0.5;
	}

	/**
	 * Calculates distance between (x1,y1) and (x2,y2)
	 * 
	 * @param x1 x Position of Point 1
	 * @param y1 y Position of Point 1
	 * @param x2 x Position of Point 1 
	 * @param y2 y Position of Point 1
	 * @return distance
	 */
	private double distance(int x1, int y1, int x2, int y2) {
		return Math.sqrt(Math.pow(x1-x2, 2)+Math.pow(y1-y2, 2));
	}

	@Override
	public void keyTyped(KeyEvent e) {
		if(e.getKeyCode()==KeyEvent.VK_CONTROL){
			if(controlDown!=true){
				controlDown = true;
				panel.repaint();
			}
		}
	}

	@Override
	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode()==KeyEvent.VK_CONTROL){
			if(controlDown!=true){
				controlDown = true;
				panel.repaint();
			}
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {
		if(e.getKeyCode()==KeyEvent.VK_CONTROL){
			if(controlDown==true){
				controlDown = false;
				panel.repaint();
			}
		}
	}
	
	@Override
	public void mouseMoved(MouseEvent e) {}

	@Override
	public void mouseEntered(MouseEvent e) {}

	@Override
	public void mouseExited(MouseEvent e) {}

	@Override
	public void actionPerformed(ActionEvent e) {}
	
	/**
	 * Shows the RightClick Menu on the Visualization Panel, with Options for
	 * the given SmartDevice clickedOn, if clickedOn is null, the RightClick
	 * Menu contains Options for creation of new SmartDevices. The Menu will be
	 * shown at the Mouse position on the VisualisationPanel.
	 * 
	 * @param clickedOn
	 *            Device which was clicked, null if no
	 */
	protected void showRightClickMenu(SmartDevice clickedOn) {
		/**
		 * Mouse Position on the VisualisationPanel
		 */
		Point mousePos = panel.getMousePosition();
		// Just execute if Mouse Position is on the Panel
		if (mousePos != null) {
			if (clickedOn == null) {
				itemCreate.setText("Create Device");
				itemCreate.setEnabled(true);
				itemDelete.setEnabled(false);
				itemDeleteSelected.setEnabled(false);
				itemCreateLink.setEnabled(false);
				if(clickedConnection.size()>0){
					if(rightClickMenu.getComponentIndex(itemCreateConnection)!=-1){
						rightClickMenu.remove(itemCreateConnection);
						itemEditConnection.removeAll();
						//List to prevent duplicate connection entries
						LinkedList<Connection> editableConnections = new LinkedList<Connection>();
						for(Pair<Connection, Pair<Port,Port>> p:clickedConnection){
							//Don't add the same connection multiple times
							if(editableConnections.contains(p.getLeft()))continue;
							editableConnections.add(p.getLeft());
							JMenuItem a = new JMenuItem(p.getLeft().getName());
							a.addActionListener(e->new ConnectionCreationDialog(p.getLeft(), controller, panel));
							itemEditConnection.add(a);
						}
						rightClickMenu.add(itemEditConnection,editCreateConnectionIndex);
						itemEditConnection.setEnabled(true);
					}
					itemCreateConnection.setEnabled(true);				
				}else{
					if(rightClickMenu.getComponentIndex(itemEditConnection)!=-1){
						rightClickMenu.remove(itemEditConnection);
						rightClickMenu.add(itemCreateConnection,editCreateConnectionIndex);
					}
					itemCreateConnection.setEnabled(false);
				}
			} else {
				itemCreate.setText("Edit Device");
				itemCreate.setEnabled(true);
				itemDelete.setEnabled(true);
				//Remove edit connection if it exist
				if(rightClickMenu.getComponentIndex(itemEditConnection)!=-1){
					rightClickMenu.remove(itemEditConnection);
					rightClickMenu.add(itemCreateConnection,editCreateConnectionIndex);
				}
				
				if(selectedDevices.contains(clickedOn)){
					itemDeleteSelected.setEnabled(true);
					itemCreateLink.setEnabled(true);
					itemCreateConnection.setEnabled(true);
				}
				else{
					itemDeleteSelected.setEnabled(false);
					itemCreateLink.setEnabled(false);
					itemCreateConnection.setEnabled(false);
				}
			}
			// Show the RightClickMenu
			rightClickMenu.show(panel, mousePos.x, mousePos.y);
			rightClickMenu.setEnabled(true);
			rightClickMenu.setVisible(true);
			mode = RIGHTCLICK_MENU;
		}
	}

	/**
	 * Creates the RightClickMenu, adds the different Labels and functionalities to the menu and adds it to the panel.
	 */
	private void initializeRightClickMenu() {
		this.rightClickMenu = new JPopupMenu();

		// Create device option
		itemCreate = new JMenuItem("Create SmartDevice");

		itemCreate.addActionListener(e -> {
			SmartDevice toEdit;
			if(clicked == null){
				toEdit = new SmartDevice("DeviceName");
				toEdit.setX(dragged_x);
				toEdit.setY(dragged_y);
			}else{
				toEdit = clicked;
			}
			SmartDeviceCreationPopUp popUp = new SmartDeviceCreationPopUp(toEdit, toEdit==clicked,controller);
			popUp.setLocationRelativeTo(panel);
			popUp.setEnabled(true);
			popUp.setVisible(true);
			mode = NOTHING;

		});

		rightClickMenu.add(itemCreate);
		
		// Create Link
		itemCreateLink = new JMenuItem("Create Link");
		
		itemCreateLink.addActionListener(e -> {
			new LinkCreationDialog(selectedDevices, controller, panel);
			if(selectedDevices.isEmpty())
				mode = NOTHING;
			else
				mode = SELECTED;
		});
		rightClickMenu.add(itemCreateLink);
		
		// Create Connection
		itemCreateConnection = new JMenuItem("Create Connection");
		itemCreateConnection.addActionListener(e->{
			LinkedList<Port> ports = new LinkedList<Port>();
			for(SmartDevice d: selectedDevices)
				ports.add(new Port(d, (short) 12));
			new ConnectionCreationDialog(ports,new SimpleLink("TestLink"), controller, panel);
			if(selectedDevices.isEmpty())
				mode = NOTHING;
			else
				mode = SELECTED;
		});
		rightClickMenu.add(itemCreateConnection);
		
		// Edit connection
		itemEditConnection = new JMenu("Edit Connection");

		// Delete device option
		itemDelete = new JMenuItem("Delete");

		itemDelete.addActionListener(e -> {
			// Delete the clicked object
				controller.deleteSmartDevice(clicked);
				clicked = null;
				controller.notifyObservers();
				mode = NOTHING;
			});

		rightClickMenu.add(itemDelete);
		
		// Delete device option
		itemDeleteSelected = new JMenuItem("Delete Selected");
		
		itemDeleteSelected.addActionListener(e -> {
			// Delete the clicked object
			for(SmartDevice c: selectedDevices)
				controller.deleteSmartDevice(c);
			selectedDevices.clear();
			clicked = null;
			controller.notifyObservers();
			mode = NOTHING;
		});
		itemDeleteSelected.setEnabled(false);
		rightClickMenu.add(itemDeleteSelected);
				
		/*
		 * Add PrintDebug option
		 */
		itemDebug = new JMenuItem("Print NetworkState");
		itemDebug.addActionListener(e -> {
			// Print Links, Devices and Connections
				for (Link l : model.getConnectionNetworks())
					System.out.println("Link: " + l.getName());
				for (SmartDevice d : model.getDevices()) {
					System.out.println("Device: " + d.getName());
					for (Port p : d.getPorts()) {
						if(p==null){
							System.out.println("Port: null");
							continue;
						}
						if(p.getConnection()==null){
							System.out.println("Connection: null");
							continue;
						}
						switch (p.getConnection().getStatus()) {
						case Connection.ACTIVE:
							System.out.println("Connection: active");
							break;
						case Connection.FINISHED:
						case Connection.TERMINATED:
							System.out.println("Connection: terminated");

							break;

						case Connection.HALTED:
						default:
							break;
						}
					}
				}
				// Print Terminating Packages
				LinkedList<Connection> terminated = new LinkedList<Connection>();

				model.getConnections()
						.stream()
						.forEach(c -> {
							if(c.getStatus() == Connection.FINISHED || c
								.getStatus() == Connection.TERMINATED)
								terminated.add(c);
							for (Packet p : c.getTerminationPackages(1000))
								System.out.println(p.getTextualRepresentation());
						});
				model.getConnections().removeAll(terminated);
				if(selectedDevices.isEmpty())
					mode = NOTHING;
				else
					mode = SELECTED;
				controller.notifyObservers();
			});
		rightClickMenu.add(itemDebug);

		this.panel.add(rightClickMenu);
	}
}
